#include <math.h>
#include <stdlib.h>
#include <assert.h>
#include "pilot.h"

// Stack of pilot id's to assure uniqueness.
static unsigned int pilot_id = 0;

// Stack of pilots - yes, they come in stacks now.
static Pilot* pilot_stack = NULL;

static void pilot_update(Pilot* pilot, const FP dt);
static void pilot_render(Pilot* pilot);

// Render the pilot.
static void pilot_render(Pilot* pilot) {
  int sprite;
  Vec2 pos;
  gl_texture* texture = pilot->ship->gfx_ship;

  pos.x = pilot->solid->pos.x;
  pos.y = pilot->solid->pos.y;

  // Get the sprite corresponding to the direction facing.
  sprite = (int)(pilot->solid->dir / (2.0*M_PI / (texture->sy * texture->sx)));

  gl_blitSprite(texture, &pos, sprite % (int)texture->sx, sprite / (int)texture->sy);
}

// Update the pilot.
static void pilot_update(Pilot* pilot, const FP dt) {
  if(pilot->solid->dir > 2*M_PI) pilot->solid->dir -= 2*M_PI;
  if(pilot->solid->dir < 0.0) pilot->solid->dir += 2*M_PI;

  // Update the solid.
  pilot->solid->update(pilot->solid, dt);

  pilot_render(pilot);
}

// ==Init pilot.===========================================
// ship  : Ship pilot is flying.
// name  : Pilot's name, if NULL, ships name will be used.
// vel   : Initial velocity.
// pos   : Initial position.
// flags : Tweaking the pilot.
// ========================================================
void pilot_init(Pilot* pilot, Ship* ship, char* name, const Vec2* vel, const Vec2* pos, const int flags) {
  pilot->id = pilot_id++; // New unique pilot id based on pilot_id.

  pilot->ship = ship;
  if(name == NULL)
    pilot->name = ship->name;
  else
    pilot->name = name;

  pilot->solid = solid_create(ship->mass, vel, pos);

  pilot->armor = ship->armor;
  pilot->shield = ship->shield;
  pilot->energy = ship->energy;

  if(flags & PILOT_PLAYER)
    pilot->think = NULL; // Players don't need to thing! :P

  pilot->update = pilot_update;
}

// Create a new pilot - Params are same as pilot_init.
Pilot* pilot_create(Ship* ship, char* name, const Vec2* vel, const Vec2* pos, const int flags) {
  Pilot* dyn = MALLOC_L(Pilot);
  assert(dyn != NULL);
  pilot_init(dyn, ship, name, vel, pos, flags);
  return dyn;
}

// Free the prisoned pilot!
void pilot_free(Pilot* pilot) {
  solid_free(pilot->solid);
  free(pilot);
  pilot = NULL;
}

